plotDispEsts = function( cds, ymin, linecol="#ff000080",
  xlab = "mean of normalized counts", ylab = "dispersion",
  log = "xy", cex = 0.45, ... )
{
  px = rowMeans( counts( cds, normalized=TRUE ) )
  sel = (px>0)
  px = px[sel]

  py = fData(cds)$dispBeforeSharing[sel]
  if(missing(ymin))
      ymin = 10^floor(log10(min(py[py>0], na.rm=TRUE))-0.1)

  plot(px, pmax(py, ymin), xlab=xlab, ylab=ylab,
    log=log, pch=ifelse(py<ymin, 6, 16), cex=cex, ... )
  xg = 10^seq( -.5, 5, length.out=100 )
  fun = function(x) { cds@dispFitCoefs[1] + cds@dispFitCoefs[2] / x }
  lines( xg, fun(xg), col=linecol, lwd=4)
}

plotMA = function(x, ylim,
  col = ifelse(x$padj>=0.1, "gray32", "red3"),
  linecol = "#ff000080",
  xlab = "mean of normalized counts", ylab = expression(log[2]~fold~change),
  log = "x", cex=0.45, ...)
{
  if (!(is.data.frame(x) && all(c("baseMean", "log2FoldChange") %in% colnames(x))))
    stop("'x' must be a data frame with columns named 'baseMean', 'log2FoldChange'.")

  x = subset(x, baseMean!=0)
  py = x$log2FoldChange
  if(missing(ylim))
      ylim = c(-1,1) * quantile(abs(py[is.finite(py)]), probs=0.99) * 1.1
  plot(x$baseMean, pmax(ylim[1], pmin(ylim[2], py)),
       log=log, pch=ifelse(py<ylim[1], 6, ifelse(py>ylim[2], 2, 16)),
       cex=cex, col=col, xlab=xlab, ylab=ylab, ylim=ylim, ...)
  abline(h=0, lwd=4, col=linecol)
}

plotDEXSeqSimple<-function (ecs, geneID, FDR = 0.1, fitExpToVar = "condition", 
    norCounts = FALSE, expression = TRUE, splicing = FALSE, displayTranscripts = FALSE, 
    names = FALSE, legend = FALSE, color = NULL, color.samples = NULL) {

    fd<-fData(ecs)[fData(ecs)$geneID==geneID,]
    op <- sum(c(expression, splicing, norCounts))
    rt <- which(featureData(ecs)$geneID == geneID)
    count <- countTableForGene(ecs, geneID, normalized = TRUE)
    rango <- seq(along = rt)
    intervals <- (0:nrow(count))/nrow(count)
    numcond <- length(unique(design(ecs, drop = FALSE)[[fitExpToVar]]))
    numexons <- nrow(count)
    each <- featureData(ecs)$padjust[rt]
    exoncol <- ifelse(each <= FDR, "#F219ED", "#CCCCCC")
    exoncol[is.na(exoncol)] <- "white"
    colorlines <- ifelse(each <= FDR, "#F219ED60", "#B3B3B360")
    colorlines[is.na(colorlines)] <- "#B3B3B360"
    colorlinesB <- ifelse(each <= FDR, "#9E109B", "#666666")
    colorlinesB[is.na(colorlinesB)] <- "#666666"
    if (!any(is.na(fd$start))) {
        sub <- data.frame(start = featureData(ecs)$start[rt], 
            end = featureData(ecs)$end[rt], chr = featureData(ecs)$chr[rt], 
            strand = featureData(ecs)$strand[rt])
        rel <- (data.frame(sub$start, sub$end)) - min(sub$start)
        rel <- rel/max(rel[, 2])
        if (displayTranscripts == TRUE & !is.null(featureData(ecs)$transcripts)) {
            transcripts <- sapply(featureData(ecs)$transcripts[rt], 
                function(x) {
                  strsplit(x, ";")
                })
            trans <- Reduce(union, transcripts)
            if (length(trans) > 40) {
                warning("This gene contains more than 40 transcripts annotated, only the first 40 will be plotted\n")
            }
            mat <- seq_len(3 + min(length(trans), 40))
            hei <- c(8, 1, 1.5, rep(1.5, min(length(trans), 40)))
        }
        else {
            mat <- 1:3
            hei <- c(5, 1, 1.5)
        }
        if (op > 1) {
            hei <- c(rep(hei[1], op - 1), hei)
            mat <- c(mat, length(mat) + seq(along = op))
        }
        hei <- c(hei, 0.2)
        mat <- c(mat, length(mat) + 1)
        layout(matrix(mat), heights = hei)
        par(mar = c(2, 4, 4, 2))
    }
    if (is.null(color)) {
        color <- rgb(colorRamp(c("#D7191C", "#FFFFBF", "#2B83BA"))(seq(0, 
            1, length.out = numcond)), maxColorValue = 255, alpha = 175)
    }
    names(color) <- sort(levels(design(ecs, drop = FALSE)[[fitExpToVar]]))
    if (expression) {
        es <- DEXSeq:::fitAndArrangeCoefs(ecs, geneID, frm = as.formula(paste("count ~", 
            fitExpToVar, "* exon")))
        if (is.null(es)) {
            warning(sprintf("glm fit failed for gene %s", geneID))
            return()
        }
        coeff <- as.matrix(t(DEXSeq:::getEffectsForPlotting(es, averageOutExpression = FALSE, 
            groupingVar = fitExpToVar)))
        coeff <- exp(coeff)
        ylimn <- c(0, max(coeff, na.rm = TRUE))
        coeff <- DEXSeq:::vst(coeff, ecs)
        DEXSeq:::drawPlot(matr = coeff, ylimn, ecs, intervals, rango, 
            textAxis = "Fitted expression", rt = rt, color = rep(color[sort(levels(design(ecs, 
                drop = FALSE)[[fitExpToVar]]))], each = numexons), 
            colorlines = colorlines)
    }
    if (!any(is.na(fd$start))) {
        par(mar = c(0, 4, 0, 2))
        plot.new()
        segments(apply((rbind(rel[rango, 2], rel[rango, 1])), 
            2, median), 0, apply(rbind(intervals[rango], intervals[rango + 
            1] - ((intervals[rango + 1] - intervals[rango]) * 
            0.2)), 2, median), 1, col = colorlinesB)
        par(mar = c(1.5, 4, 0, 2))
        DEXSeq:::drawGene(min(sub$start), max(sub$end), tr = sub, rango, 
            exoncol = exoncol, names, trName = "Gene model", 
            cex = 0.8)
        if (!is.null(featureData(ecs)$transcripts)) {
            if (displayTranscripts) {
                for (i in seq_len(min(length(trans), 40))) {
                  logicexons <- sapply(transcripts, function(x) {
                    length(which(x == trans[i]))
                  })
                  tr <- data.frame(featureData(ecs)$start[rt][logicexons == 
                    1], featureData(ecs)$end[rt][logicexons == 
                    1])
                  DEXSeq:::drawGene(min(sub$start), max(sub$end), tr = tr, 
                    rango, exoncol = NULL, names, trName = trans[i], 
                    cex = 0.8)
                }
            }
        }
        axis(1, at = round(seq(min(sub$start), max(sub$end), 
            length.out = 10)), labels = round(seq(min(sub$start), 
            max(sub$end), length.out = 10)), pos = 0, lwd.ticks = 0.2, 
            padj = -0.7)
    }
    if (legend) {
        mtext(paste(geneID, unique(featureData(ecs)$strand[rt])), 
            side = 3, adj = 0.25, padj = 1.5, line = 0, outer = TRUE, 
            cex = 1.5)
        posforlegend <- seq(0.7, 0.9, length.out = numcond)
        for (i in seq(along = color)) {
            mtext(names(color[i]), side = 3, adj = posforlegend[i], 
                padj = 1.5, line = 0, outer = TRUE, col = color[i] 
                )
        }
    } else {
        mtext(paste(geneID, unique(featureData(ecs)$strand[rt])), 
            side = 3, adj = 0.5, padj = 1.5, line = 0, outer = TRUE, 
            cex = 1.5)
    }
}

